#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
The app.py module defines a main() function that includes the logic for the `font-line` command line executable.

This command line executable modifies the line spacing metrics in one or more fonts that are provided as command
line arguments to the executable.  It currently supports .ttf and .otf font builds.
"""

import sys

from commandlines import Command
from standardstreams import stdout, stderr

from fontline import settings
from fontline.commands import get_font_report, modify_linegap_percent, get_linegap_percent_filepath
from fontline.utilities import file_exists, is_supported_filetype

# TODO: support integer addition and subtraction to the metrics values through new sub-commands
# TODO: JSON formatted metrics output
# TODO: add report line items for 'exceeds ymax/ymin values' across each of the three metrics


def main():
    """Defines the logic for the `font-line` command line executable"""
    c = Command()

    if c.does_not_validate_missing_args():
        stderr("[font-line] ERROR: Please include one or more arguments with your command.")
        sys.exit(1)

    if c.is_help_request():
        stdout(settings.HELP)
        sys.exit(0)
    elif c.is_version_request():
        stdout(settings.VERSION)
        sys.exit(0)
    elif c.is_usage_request():
        stdout(settings.USAGE)
        sys.exit(0)

    # REPORT sub-command
    if c.subcmd == "report":
        if c.argc < 2:
            stderr("[font-line] ERROR: Missing file path argument(s) after the report subcommand.")
            sys.exit(1)
        else:
            for fontpath in c.argv[1:]:
                # test for existence of file on path
                if file_exists(fontpath):
                    # test that filepath includes file of a supported file type
                    if is_supported_filetype(fontpath):
                        stdout(get_font_report(fontpath))
                    else:
                        stderr("[font-line] ERROR: '" + fontpath + "' does not appear to be a supported font file type.")
                else:
                    stderr("[font-line] ERROR: '" + fontpath + "' does not appear to be a valid filepath.")
    # PERCENT sub-command
    elif c.subcmd == "percent":
        if c.argc < 3:
            stderr("[font-line] ERROR: Not enough arguments.")
            sys.exit(1)
        else:
            percent = c.argv[1]
            # test the percent integer argument
            try:
                percent_int = int(percent)  # test that the argument can be cast to an integer value
                if percent_int <= 0:
                    stderr("[font-line] ERROR: Please enter a percent value that is greater than zero.")
                    sys.exit(1)
                if percent_int > 100:
                    stdout("[font-line] Warning: You entered a percent value over 100%. Please confirm that this is "
                           "your intended metrics modification.")
            except ValueError:
                stderr("[font-line] ERROR: You entered '" + percent + "'. This argument needs to be an integer value.")
                sys.exit(1)
            for fontpath in c.argv[2:]:
                if file_exists(fontpath):
                    if is_supported_filetype(fontpath):
                        if modify_linegap_percent(fontpath, percent) is True:
                            outpath = get_linegap_percent_filepath(fontpath, percent)
                            stdout("[font-line] '" + fontpath + "' successfully modified to '" + outpath + "'.")
                        else:  # pragma: no cover
                            stderr("[font-line] ERROR: Unsuccessful modification of '" + fontpath + "'.")
                    else:
                        stderr("[font-line] ERROR: '" + fontpath + "' does not appear to be a supported font file type.")
                else:
                    stderr("[font-line] ERROR: '" + fontpath + "' does not appear to be a valid filepath.")
    else:
        stderr("[font-lines] ERROR: You used an unsupported argument to the executable. Please review the"
               " `font-line --help` documentation and try again.")
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ------------------------------------------------------------------------------
# Application Name
# ------------------------------------------------------------------------------
app_name = 'font-line'

# ------------------------------------------------------------------------------
# Version Number
# ------------------------------------------------------------------------------
major_version = "0"
minor_version = "7"
patch_version = "0"

# ------------------------------------------------------------------------------
# Help String
# ------------------------------------------------------------------------------

HELP = """====================================================
font-line
Copyright 2017 Christopher Simpkins
MIT License
Source: https://github.com/source-foundry/font-line
====================================================

ABOUT

font-line is a font vertical metrics reporting and line spacing modification tool.

SUB-COMMANDS

  percent - adjust font line spacing to % of UPM value
  report - generate report of font metrics and derived values

OPTIONS

  -h | --help        display application help
  -v | --version     display application version
       --usage       display usage information

USAGE

 $ font-line report [fontpath 1] <fontpath ...>
 $ font-line percent [integer] [fontpath 1] <fontpath ...>
 $ font-line [-v|-h] [--help|--usage|--version]

Reports are sent to the standard output stream with the `report` sub-command.

Original font files are not modified when you use the `percent` sub-command.  Instead a new file write occurs on a path
that is displayed in the standard output stream following the modification.  No modifications are made to the original
glyph set or hints associated with the original font build.

For more information about the OpenType table modifications that occur, please see the project documentation at:

https://github.com/source-foundry/font-line"""

# ------------------------------------------------------------------------------
# Version String
# ------------------------------------------------------------------------------

VERSION = "font-line v" + major_version + "." + minor_version + "." + patch_version


# ------------------------------------------------------------------------------
# Usage String
# ------------------------------------------------------------------------------

USAGE = """
Usage: font-line [subcommand] <arg> [font path 1] <font path 2> <font path ...>"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os.path

from fontTools import ttLib
from fontline.utilities import get_sha1

from standardstreams import stderr


def get_font_report(fontpath):
    tt = ttLib.TTFont(fontpath)

    # Vertical metrics values as integers
    os2_typo_ascender = tt['OS/2'].sTypoAscender
    os2_typo_descender = tt['OS/2'].sTypoDescender
    os2_win_ascent = tt['OS/2'].usWinAscent
    os2_win_descent = tt['OS/2'].usWinDescent
    os2_typo_linegap = tt['OS/2'].sTypoLineGap
    os2_x_height = tt['OS/2'].sxHeight
    os2_cap_height = tt['OS/2'].sCapHeight
    hhea_ascent = tt['hhea'].ascent
    hhea_descent = tt['hhea'].descent
    hhea_linegap = tt['hhea'].lineGap
    ymax = tt['head'].yMax
    ymin = tt['head'].yMin
    units_per_em = tt['head'].unitsPerEm

    # Calculated values
    os2_typo_total_height = os2_typo_ascender + -(os2_typo_descender)
    os2_win_total_height = os2_win_ascent + os2_win_descent
    hhea_total_height = hhea_ascent + -(hhea_descent)
    typo_to_upm = 1.0 * (os2_typo_linegap + os2_typo_total_height) / units_per_em
    winascdesc_to_upm = 1.0 * os2_win_total_height / units_per_em
    hheaascdesc_to_upm = 1.0 * hhea_total_height / units_per_em

    # The file path header
    report = [" "]
    report.append("=== " + fontpath + " ===")
    namerecord_list = tt['name'].names
    # The version string
    for needle in namerecord_list:
        if needle.nameID == 5:
            report.append(needle.toStr())
            break
    # The SHA1 string
    report.append("SHA1: " + get_sha1(fontpath))
    report.append("")
    # The vertical metrics strings
    report.append("--- Metrics ---")
    report.append("[head] Units per Em: \t{}".format(units_per_em))
    report.append("[head] yMax: \t\t{}".format(ymax))
    report.append("[head] yMin: \t\t{}".format(ymin))
    report.append("[OS/2] CapHeight: \t{}".format(os2_cap_height))
    report.append("[OS/2] xHeight: \t{}".format(os2_x_height))
    report.append("[OS/2] TypoAscender: \t{}".format(os2_typo_ascender))
    report.append("[OS/2] TypoDescender: \t{}".format(os2_typo_descender))
    report.append("[OS/2] WinAscent: \t{}".format(os2_win_ascent))
    report.append("[OS/2] WinDescent: \t{}".format(os2_win_descent))
    report.append("[hhea] Ascent: \t\t{}".format(hhea_ascent))
    report.append("[hhea] Descent: \t{}".format(hhea_descent))
    report.append("")
    report.append("[hhea] LineGap: \t{}".format(hhea_linegap))
    report.append("[OS/2] TypoLineGap: \t{}".format(os2_typo_linegap))
    report.append("")
    report.append("--- Height Calculations by Table Values ---")
    report.append("[OS/2] TypoAscender to TypoDescender: \t{}".format(os2_typo_total_height))
    report.append("[OS/2] WinAscent to WinDescent: \t{}".format(os2_win_total_height))
    report.append("[hhea] Ascent to Descent: \t\t{}".format(hhea_total_height))
    report.append("")
    report.append("--- Delta Values ---")
    report.append("WinAscent to TypoAscender: \t{}".format(os2_win_ascent - os2_typo_ascender))
    report.append("Ascent to TypoAscender: \t{}".format(hhea_ascent - os2_typo_ascender))
    report.append("WinDescent to TypoDescender: \t{}".format(os2_win_descent - -(os2_typo_descender)))
    report.append("Descent to TypoDescender: \t{}".format(os2_typo_descender - hhea_descent))
    report.append("")
    report.append("--- Ratios ---")
    report.append("(Typo Asc + Desc + Linegap) / UPM: \t{0:.3g}".format(typo_to_upm))
    report.append("(winAsc + winDesc) / UPM: \t\t{0:.3g}".format(winascdesc_to_upm))
    report.append("(hhea Asc + Desc) / UPM: \t\t{0:.3g}".format(hheaascdesc_to_upm))

    return "\n".join(report)


def modify_linegap_percent(fontpath, percent):
    try:
        tt = ttLib.TTFont(fontpath)

        # get observed start values from the font
        os2_typo_ascender = tt['OS/2'].sTypoAscender
        os2_typo_descender = tt['OS/2'].sTypoDescender
        os2_typo_linegap = tt['OS/2'].sTypoLineGap
        hhea_ascent = tt['hhea'].ascent
        hhea_descent = tt['hhea'].descent
        units_per_em = tt['head'].unitsPerEm

        # calculate necessary delta values
        os2_typo_ascdesc_delta = os2_typo_ascender + -(os2_typo_descender)
        hhea_ascdesc_delta = hhea_ascent + -(hhea_descent)

        # define percent UPM from command line request
        factor = 1.0 * int(percent) / 100

        # define line spacing units
        line_spacing_units = int(factor * units_per_em)

        # define total height as UPM + line spacing units
        total_height = line_spacing_units + units_per_em

        # height calculations for adjustments
        delta_height = total_height - hhea_ascdesc_delta
        upper_lower_add_units = int(0.5 * delta_height)

        # redefine hhea linegap to 0 in all cases
        hhea_linegap = 0

        # Define metrics based upon original design approach in the font
        # Google metrics approach
        if os2_typo_linegap == 0 and (os2_typo_ascdesc_delta > units_per_em):
            # define values
            os2_typo_ascender += upper_lower_add_units
            os2_typo_descender -= upper_lower_add_units
            hhea_ascent += upper_lower_add_units
            hhea_descent -= upper_lower_add_units
            os2_win_ascent = hhea_ascent
            os2_win_descent = -hhea_descent
        # Adobe metrics approach
        elif os2_typo_linegap == 0 and (os2_typo_ascdesc_delta == units_per_em):
            hhea_ascent += upper_lower_add_units
            hhea_descent -= upper_lower_add_units
            os2_win_ascent = hhea_ascent
            os2_win_descent = -hhea_descent
        else:
            os2_typo_linegap = line_spacing_units
            hhea_ascent = int(os2_typo_ascender + 0.5 * os2_typo_linegap)
            hhea_descent = -(total_height - hhea_ascent)
            os2_win_ascent = hhea_ascent
            os2_win_descent = -hhea_descent

        # define updated values from above calculations
        tt['hhea'].lineGap = hhea_linegap
        tt['OS/2'].sTypoAscender = os2_typo_ascender
        tt['OS/2'].sTypoDescender = os2_typo_descender
        tt['OS/2'].sTypoLineGap = os2_typo_linegap
        tt['OS/2'].usWinAscent = os2_win_ascent
        tt['OS/2'].usWinDescent = os2_win_descent
        tt['hhea'].ascent = hhea_ascent
        tt['hhea'].descent = hhea_descent

        tt.save(get_linegap_percent_filepath(fontpath, percent))
        return True
    except Exception as e:  # pragma: no cover
        stderr("[font-line] ERROR: Unable to modify the line spacing in the font file '" + fontpath + "'. " + str(e))
        sys.exit(1)


def get_linegap_percent_filepath(fontpath, percent):
    fontpath_list = os.path.split(fontpath)
    font_dirname = fontpath_list[0]
    font_basename = fontpath_list[1]
    basepath_list = font_basename.split(".")
    outfile_basename = basepath_list[0] + "-linegap" + percent + "." + basepath_list[1]
    return os.path.join(font_dirname, outfile_basename)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import hashlib


def file_exists(filepath):
    """Tests for existence of a file on the string filepath"""
    if os.path.exists(filepath) and os.path.isfile(filepath):  # test that exists and is a file
        return True
    else:
        return False


def is_supported_filetype(filepath):
    """Tests file extension to determine appropriate file type for the application"""
    testpath = filepath.lower()
    if testpath.endswith('.ttf') or testpath.endswith('.otf'):
        return True
    else:
        return False


def get_sha1(filepath):
    with open(filepath, 'rb') as bin_reader:
        data = bin_reader.read()
    return hashlib.sha1(data).hexdigest()



import os
import re
from setuptools import setup, find_packages


def docs_read(fname):
    return open(os.path.join(os.path.dirname(__file__), 'docs', fname)).read()


def version_read():
    settings_file = open(os.path.join(os.path.dirname(__file__), 'lib', 'fontline', 'settings.py')).read()
    major_regex = """major_version\s*?=\s*?["']{1}(\d+)["']{1}"""
    minor_regex = """minor_version\s*?=\s*?["']{1}(\d+)["']{1}"""
    patch_regex = """patch_version\s*?=\s*?["']{1}(\d+)["']{1}"""
    major_match = re.search(major_regex, settings_file)
    minor_match = re.search(minor_regex, settings_file)
    patch_match = re.search(patch_regex, settings_file)
    major_version = major_match.group(1)
    minor_version = minor_match.group(1)
    patch_version = patch_match.group(1)
    if len(major_version) == 0:
        major_version = 0
    if len(minor_version) == 0:
        minor_version = 0
    if len(patch_version) == 0:
        patch_version = 0
    return major_version + "." + minor_version + "." + patch_version


setup(
    name='font-line',
    version=version_read(),
    description='A font vertical metrics reporting and line spacing adjustment tool',
    long_description=(docs_read('README.rst')),
    url='https://github.com/source-foundry/font-line',
    license='MIT license',
    author='Christopher Simpkins',
    author_email='chris@sourcefoundry.org',
    platforms=['any'],
    packages=find_packages("lib"),
    package_dir={'': 'lib'},
    install_requires=['commandlines', 'standardstreams', 'fontTools'],
    entry_points={
        'console_scripts': [
            'font-line = fontline.app:main'
        ],
    },
    keywords='font,typeface,fonts,spacing,line spacing,spaces,vertical metrics,metrics,type',
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
